local AddonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary

local math = math

c.Init(a)

------------------------------------------------------------------------ common
local function needsInner()
	return not s.Buff(c.GetID("Inner Fire"), "player")
		and not s.Buff(c.GetID("Inner Will"), "player")
end

c.AddOptionalSpell("Power Word: Fortitude", nil, {
	NoRangeCheck = true,
	CheckFirst = function()
		return c.RaidBuffNeeded(c.STAMINA_BUFFS)
	end
})

c.AddOptionalSpell("Shadow Protection", nil, {
	NoRangeCheck = true,
	CheckFirst = function()
		return c.RaidBuffNeeded(c.SHADOW_RESISTANCE_BUFFS)
	end
})

c.AddOptionalSpell("Flash Heal", "under Surge of Light", {
	NoRangeCheck = true,
	NoStopChannel = true,
	CheckFirst = function()
		return c.HasBuff("Surge of Light")
	end
})

c.AddOptionalSpell("Binding Heal", nil, {
	NoRangeCheck = true,
	CheckFirst = function()
		local target = nil
		if a.GetConfig("mouseover") then
			target = "mouseover"
		else
			target = s.UnitSelection()
		end
		return not UnitIsUnit(target, "player")
			and UnitIsFriend(target, "player")
			and s.HealthDamage(target) > 30000
			and s.HealthDamage("player") > 30000
	end
})

c.AddOptionalSpell("Shadowfiend", "for Mana", {
	NoStopChannel = true,
	CheckFirst = function()
		return s.PowerPercent("player") < 70
	end
})

c.AddOptionalSpell("Inner Fire", nil, {
	Buff = "Inner Fire",
	BuffUnit = "player",
})

c.AddOptionalSpell("Inner Fire", "unless Inner Will", {
	CheckFirst = needsInner,
})

c.AddOptionalSpell("Inner Will", nil, {
	Buff = "Inner Will",
	BuffUnit = "player",
})

c.AddOptionalSpell("Inner Will", "unless Inner Fire", {
	CheckFirst = needsInner,
})

c.AddOptionalSpell("Desperate Prayer", nil, {
	NoStopChannel = true,
	NoRangeCheck = true,
	CheckFirst = function()
		return s.HealthPercent("player") < 70
	end
})

-------------------------------------------------------------------- discipline
c.AddOptionalSpell("Pennance")

c.AddOptionalSpell("Inner Focus", nil, {
	NoGCD = true,
})

c.AddOptionalSpell("Power Word: Shield", nil, {
	MyBuff = "Power Word: Shield",
	NoStopChannel = true,
	CheckFirst = function(z)
		if a.GetConfig("mouseover") then
			z.Unit = "mouseover"
		else
			z.Unit = nil
		end
		return not s.Debuff(
			c.GetID("Weakened Soul"), z.Unit, c.GetBusyTime())
	end
})

-------------------------------------------------------------------------- holy
c.AddOptionalSpell("Lightwell", nil, {
	NoRangeCheck = true,
	NotIfActive = true,
})

local function serendipityCheck(z)
	if not c.HasTalent("Serendipity") then
		return false
	end
	
	local stacks = c.GetBuffStack("Serendipity 1")
	if stacks == 0 then
		stacks = c.GetBuffStack("Serendipity 2")
	end
	if stacks < 2
		and c.IsCasting("Binding Heal") or c.IsCasting("Flash Heal") then
		
		stacks = stacks + 1
	elseif c.IsCasting("Greater Heal") or c.IsCasting("Prayer of Healing") then
		stacks = 0
	end
	
	if stacks == 1 then
		z.FlashSize = s.FlashSizePercent() / 2
	else
		z.FlashSize = nil
	end
	return stacks > 0
end

c.AddOptionalSpell("Greater Heal", "under Serendipity", {
	NoRangeCheck = true,
	CheckFirst = serendipityCheck,
})

c.AddOptionalSpell("Prayer of Healing", "under Serendipity", {
	NoRangeCheck = true,
	CheckFirst = serendipityCheck,
})

c.AddOptionalSpell("Chakra", nil, {
	NoGCD = true,
	CheckFirst = function(z)
		return not c.HasBuff("Chakra: Serenity")
			and not c.HasBuff("Chakra: Sanctuary")
	end
})

local holyWords = c.GetIDs(
	"Holy Word: Chastise",
	"Holy Word: Serenity",
	"Holy Word: Sanctuary"
)

local function holyWordOverride(z)
--	if c.HasBuff("Chakra: Serenity") then
--		return c.GetCooldown("Holy Word: Serenity") == 0
--	else
	if c.HasBuff("Chakra: Sanctuary") then
		return c.GetCooldown("Holy Word: Sanctuary") == 0
	else
		return false
	end
end

c.AddOptionalSpell("Holy Word: Chastise", nil, {
	FlashID = holyWords,
	Override = holyWordOverride,
})

c.AddOptionalSpell("Holy Word: Serenity", nil, {
	FlashID = holyWords,
	Override = holyWordOverride,
})

c.AddOptionalSpell("Holy Word: Sanctuary", nil, {
	FlashID = holyWords,
	Override = holyWordOverride,
})

c.AddOptionalSpell("Prayer of Mending", nil, {
	NoRangeCheck = true,
	CheckFirst = function(z)
		return not a.PrayerOfMendingTarget
			or s.MyBuffDuration(z.ID, a.PrayerOfMendingTarget) < c.GetBusyTime()
			or s.MyBuffStack(z.ID, a.PrayerOfMendingTarget) < 5
	end
})

------------------------------------------------------------------------ shadow
c.AddOptionalSpell("Shadowform", nil, {
	Type = "form",
})

c.AddOptionalSpell("Vampiric Embrace", nil, {
	Buff = "Vampiric Embrace",
	BuffUnit = "player",
})

c.AddOptionalSpell("Shadowfiend", nil, {
	NoStopChannel = true,
})

c.AddSpell("Shadow Word: Death", nil, {
	NoStopChannel = true,
	CheckFirst = function(z)
		if s.HealthPercent() > 25 then
			z.FlashColor = "yellow"
			z.Continue = true
			return s.PowerPercent("player") < 30
		else
			z.FlashColor = nil
			z.Continue = nil
			return true
		end
	end
})

c.AddSpell("Vampiric Touch", nil, {
	MyDebuff = "Vampiric Touch",
	NotIfActive = true,
	NoStopChannel = true,
})
c.ManageDotRefresh("Vampiric Touch", 3)

c.AddOptionalSpell("Devouring Plague", nil, {
	MyDebuff = "Devouring Plague",
	NoStopChannel = true,
})
c.ManageDotRefresh("Devouring Plague", 3)

c.AddSpell("Shadow Word: Pain", nil, {
	MyDebuff = "Shadow Word: Pain",
})

c.AddSpell("Mind Blast", nil, {
	NotIfActive = true,
	NoStopChannel = true,
})

c.AddSpell("Mind Blast", "at 3 Orbs", {
	NotIfActive = true,
	NoStopChannel = true,
	CheckFirst = function()
		return c.GetBuffStack("Shadow Orb") == 3
	end
})

c.AddSpell("Mind Blast", "for Empowered Shadow", {
	NotIfActive = true,
	CheckFirst = function()
		return c.HasBuff("Shadow Orb")
	end
})

c.ManageDotRefresh("Mind Flay", 1)

--local function shouldFlayToRefresh(duration)
--	local refreshDelay = (c.GetSpell("Mind Flay").EarlyRefresh or 1) + .1
--	return duration > refreshDelay
--		and duration < refreshDelay + math.max(1, c.GetCastTime("Mind Blast"))
--end
--
--c.AddSpell("Mind Flay", "for SW:P", {
--	CheckFirst = function()
--		return shouldFlayToRefresh(c.GetMyDebuffDuration("Shadow Word: Pain"))
--	end
--})
--
--c.AddSpell("Mind Flay", "to save Dark Evangelism", {
--	CheckFirst = function()
--		return shouldFlayToRefresh(c.GetBuffDuration("Dark Evangelism"))
--	end
--})

c.AddSpell("Mind Flay", "for Dark Evangelism", {
	CheckFirst = function(z)
		if not c.HasTalent("Evangelism") then
			return false
		end
		
		local stacks = c.GetBuffStack("Dark Evangelism")
		if stacks < 5 and c.IsCasting("Mind Flay") then
			local remaining = s.Channeling(z.ID, "player")
			if remaining then
				stacks = stacks
					+ remaining / c.GetSpell("Mind Flay").EarlyRefresh
			else
				stacks = stacks + 3
			end
		end
		return stacks < 5
	end
})

c.AddSpell("Mind Flay", "for Empowered Shadow", {
	CheckFirst = function()
		return not c.HasBuff("Empowered Shadow")
			and not c.IsCasting("Mind Blast")
	end
})

c.AddOptionalSpell("Dark Archangel", nil, {
	NoGCD = true,
	CheckFirst = function()
		return c.GetBuffStack("Dark Evangelism") == 5
	end
})

c.AddOptionalSpell("Dispursion", nil, {
	CheckFirst = function()
		return s.PowerPercent("player") < 64
	end
})
